@@ -6,7 +6,8 @@ class NN(torch.nn.Module):
 
     def __init__(self):
         super(NN, self).__init__()
-
+        # torch.nn.Module has variable training
+        self.inputs = None
         self._input_transform = None
         self._output_transform = None
 
